@@ -40,6 +40,31 @@ module type Additional_helpers = sig
   val esequence : (expression list -> expression) with_loc
   val ppat_tuple_opt : (pattern list -> pattern option) with_loc
   val pexp_tuple_opt : (expression list -> expression option) with_loc
+
+  val pexp_fun :
+    (arg_label -> expression option -> pattern -> expression -> expression)
+    with_loc
+  (** [pexp_fun] can be used to create function expressions. It will check if
+      the function's body is itself a function expression and if so it will
+      coalesce the arguments.
+
+      For example, if we have [pexp_fun Nolabel None (var "x") f] and [f] is
+      [fun y -> x + y] then the function expression returned will be
+      [fun x y -> x + y] and not [fun x -> y -> x + y]. However, it will be more
+      efficient to create maximum arity functions directly with
+      {! pexp_function}. *)
+
+  val pexp_function :
+    (function_param list ->
+    type_constraint option ->
+    function_body ->
+    expression)
+    with_loc
+
+  val pexp_function_cases : (Import.cases -> expression) with_loc
+  (** [pexp_function_cases] builds an expression in the shape
+      [function C1 -> E1 | ...]. *)
+
   val pconstruct : constructor_declaration -> pattern option -> pattern
   val econstruct : constructor_declaration -> expression option -> expression
 
@@ -59,6 +84,10 @@ module type Additional_helpers = sig
   (** [plist ~loc [pat1; pat2; pat3]] produces the list pattern
       [[pat1; pat2; pat3]]. *)
 
+  val value_binding :
+    (pat:Import.pattern -> expr:Import.expression -> Import.value_binding)
+    with_loc
+
   val pstr_value_list :
     loc:Location.t ->
     Asttypes.rec_flag ->
