@@ -1,8 +1,7 @@
+(** Small helper to find out who is the caller of a function *)
+
 open! Import
-
-module Printexc = Caml.Printexc
-
-(* Small helper to find out who is the caller of a function *)
+module Printexc = Stdlib.Printexc
 
 type t = Printexc.location option
 
@@ -11,20 +10,14 @@ let get ~skip =
   let stack = Printexc.get_callstack 16 in
   let len = Printexc.raw_backtrace_length stack in
   let rec loop pos =
-    if pos = len then
-      None
+    if pos = len then None
     else
       match
         Printexc.get_raw_backtrace_slot stack pos
-        |> Printexc.convert_raw_backtrace_slot
-        |> Printexc.Slot.location
+        |> Printexc.convert_raw_backtrace_slot |> Printexc.Slot.location
       with
       | None -> None
       | Some loc ->
-        if List.mem ~set:skip loc.filename then
-          loop (pos + 1)
-        else
-          Some loc
+          if List.mem ~set:skip loc.filename then loop (pos + 1) else Some loc
   in
   loop 0
-;;
