@@ -109,11 +109,13 @@ val add :
   ?str_type_ext:(structure, type_extension) Generator.t ->
   ?str_exception:(structure, type_exception) Generator.t ->
   ?str_module_type_decl:(structure, module_type_declaration) Generator.t ->
+  ?str_module_binding:(structure, module_binding) Generator.t ->
   ?sig_type_decl:(signature, rec_flag * type_declaration list) Generator.t ->
   ?sig_class_type_decl:(signature, class_type_declaration list) Generator.t ->
   ?sig_type_ext:(signature, type_extension) Generator.t ->
   ?sig_exception:(signature, type_exception) Generator.t ->
   ?sig_module_type_decl:(signature, module_type_declaration) Generator.t ->
+  ?sig_module_decl:(signature, module_declaration) Generator.t ->
   ?extension:(loc:Location.t -> path:string -> core_type -> expression) ->
   string ->
   t
@@ -137,11 +139,13 @@ val add_alias :
   ?str_type_ext:t list ->
   ?str_exception:t list ->
   ?str_module_type_decl:t list ->
+  ?str_module_binding:t list ->
   ?sig_type_decl:t list ->
   ?sig_class_type_decl:t list ->
   ?sig_type_ext:t list ->
   ?sig_exception:t list ->
   ?sig_module_type_decl:t list ->
+  ?sig_module_decl:t list ->
   t list ->
   t
 (** [add_alias name set] add an alias. When the user write the alias, all the
@@ -151,5 +155,4 @@ val add_alias :
 
 val ignore : t -> unit
 (** Ignore a deriver. So that one can write:
-    [Deriving.add ... |>
-    Deriving.ignore] *)
+    [Deriving.add ... |> Deriving.ignore] *)
