@@ -25,6 +25,7 @@ let styler = ref None
 let output_metadata_filename = ref None
 let corrected_suffix = ref ".ppx-corrected"
 let keywords = ref None
+let raise_embedded_errors_flag = ref false
 
 let ghost =
   object
@@ -32,6 +33,17 @@ let ghost =
     method! location loc = { loc with loc_ghost = true }
   end
 
+let raise_embedded_errors =
+  object
+    inherit Ast_traverse.map as super
+
+    method! extension extension =
+      if !raise_embedded_errors_flag then
+        extension |> Location.Error.of_extension
+        |> Option.iter ~f:Location.Error.raise;
+      super#extension extension
+  end
+
 let chop_prefix ~prefix x =
   if String.is_prefix ~prefix x then
     Some (String.drop_prefix x (String.length prefix))
@@ -696,7 +708,9 @@ let map_structure_gen st ~tool_name ~hook ~expect_mismatch_handler ~input_name
       ~dropped_so_far:Attribute.dropped_so_far_structure ~hook
       ~expect_mismatch_handler ~input_name ~embed_errors
   in
-  st |> lint lint_errors |> cookies_and_check |> with_errors (List.rev errors)
+  st |> lint lint_errors |> cookies_and_check
+  |> with_errors (List.rev errors)
+  |> raise_embedded_errors#structure
 
 let map_structure st =
   match
@@ -772,7 +786,9 @@ let map_signature_gen sg ~tool_name ~hook ~expect_mismatch_handler ~input_name
       ~dropped_so_far:Attribute.dropped_so_far_signature ~hook
       ~expect_mismatch_handler ~input_name ~embed_errors
   in
-  sg |> lint lint_errors |> cookies_and_check |> with_errors (List.rev errors)
+  sg |> lint lint_errors |> cookies_and_check
+  |> with_errors (List.rev errors)
+  |> raise_embedded_errors#signature
 
 let map_signature sg =
   match
@@ -1141,8 +1157,12 @@ let process_file (kind : Kind.t) fn ~input_name ~relocate ~use_compiler_pprint
           (if use_compiler_pprint then Utils.print_as_compiler_source ppf ast
            else
              match ast with
-             | Intf ast -> Pprintast.signature ppf ast
-             | Impl ast -> Pprintast.structure ppf ast);
+             | Intf ast ->
+                 Pprintast.signature ppf
+                   (Clean.remove_migration_attributes#signature ast)
+             | Impl ast ->
+                 Pprintast.structure ppf
+                   (Clean.remove_migration_attributes#structure ast));
           let null_ast =
             match ast with Intf [] | Impl [] -> true | _ -> false
           in
@@ -1320,6 +1340,12 @@ let shared_args =
        applied before all impl and intf." );
     ("-cookie", Arg.String set_cookie, "NAME=EXPR Set the cookie NAME to EXPR");
     ("--cookie", Arg.String set_cookie, " Same as -cookie");
+    ( "-raise-embedded-errors",
+      Arg.Set raise_embedded_errors_flag,
+      " Raise the first embedded error found in the processed AST" );
+    ( "-allow-deriving-end",
+      Arg.Bool (( := ) Code_matcher.allow_deriving_end),
+      " Whether to allow [@@@deriving.end], which will soon be deprecated." );
   ]
 
 let () =
@@ -1562,3 +1588,19 @@ let enable_checks () =
 let enable_location_check () = perform_locations_check := true
 let disable_location_check () = perform_locations_check := false
 let map_structure st = map_structure st
+
+let () =
+  register_transformation "expand_inline"
+    ~rules:
+      [
+        Context_free.Rule.attr_str_floating_expect_and_expand
+          (Attribute.Floating.declare "expand_inline" Structure_item
+             Ast_pattern.(pstr __)
+             Fn.id)
+          (fun ~ctxt:_ items -> Utils.prettify_odoc_attributes#structure items);
+        Context_free.Rule.attr_sig_floating_expect_and_expand
+          (Attribute.Floating.declare "expand_inline" Signature_item
+             Ast_pattern.(psig __)
+             Fn.id)
+          (fun ~ctxt:_ items -> Utils.prettify_odoc_attributes#signature items);
+      ]
