@@ -21,60 +21,81 @@ let assert_no_attributes ~path ~prefix =
     (fqn_longident' path (prefix ^ "attributes"))
 
 let gen_combinator_for_constructor ?wrapper path ~prefix cd =
-  match cd.pcd_args with
-  | Pcstr_record _ -> failwith "Pcstr_record not supported"
-  | Pcstr_tuple cd_args ->
-      let args = List.mapi cd_args ~f:(fun i _ -> sprintf "x%d" i) in
-      let funcs = List.mapi cd_args ~f:(fun i _ -> sprintf "f%d" i) in
-      let pat =
-        Pat.construct
-          (Loc.mk (fqn_longident path cd.pcd_name.txt))
-          (match args with
-          | [] -> None
-          | [ x ] -> Some (pvar x)
-          | _ -> Some (Pat.tuple (List.map args ~f:pvar)))
-      in
-      let exp, _ = apply_parsers funcs (List.map args ~f:evar) cd_args in
-      let expected = without_prefix ~prefix cd.pcd_name.txt in
-      let body =
-        M.expr
-          {|match x with
+  let exp, pat, _args, funcs =
+    match cd.pcd_args with
+    | Pcstr_record r ->
+        let args = List.map ~f:(fun p -> sprintf "%s" p.pld_name.txt) r in
+        let funcs = List.map ~f:(fun p -> sprintf "f%s" p.pld_name.txt) r in
+        let typs = List.map ~f:(fun p -> p.pld_type) r in
+        let pat =
+          Pat.construct
+            (Loc.mk (fqn_longident path cd.pcd_name.txt))
+            (match args with
+            | [] -> None
+            | [ x ] -> Some (pvar x)
+            | _ ->
+                Some
+                  (Pat.record
+                     (List.map r ~f:(fun v ->
+                          ( { loc = v.pld_loc; txt = Lident v.pld_name.txt },
+                            pvar v.pld_name.txt )))
+                     Closed))
+        in
+        let exp, _ = apply_parsers funcs (List.map args ~f:evar) typs in
+        (exp, pat, args, funcs)
+    | Pcstr_tuple cd_args ->
+        let args = List.mapi cd_args ~f:(fun i _ -> sprintf "x%d" i) in
+        let funcs = List.mapi cd_args ~f:(fun i _ -> sprintf "f%d" i) in
+        let pat =
+          Pat.construct
+            (Loc.mk (fqn_longident path cd.pcd_name.txt))
+            (match args with
+            | [] -> None
+            | [ x ] -> Some (pvar x)
+            | _ -> Some (Pat.tuple (List.map args ~f:pvar)))
+        in
+        let exp, _ = apply_parsers funcs (List.map args ~f:evar) cd_args in
+        (exp, pat, args, funcs)
+  in
+  let expected = without_prefix ~prefix cd.pcd_name.txt in
+  let body =
+    M.expr
+      {|match x with
           | %a -> ctx.matched <- ctx.matched + 1; %a
           | _ -> fail loc %S|}
-          A.patt pat A.expr exp expected
-      in
-      let body =
-        match wrapper with
-        | None -> body
-        | Some (path, prefix, has_attrs) ->
-            let body =
-              M.expr
-                {|let loc = x.%a in
+      A.patt pat A.expr exp expected
+  in
+  let body =
+    match wrapper with
+    | None -> body
+    | Some (path, prefix, has_attrs) ->
+        let body =
+          M.expr
+            {|let loc = x.%a in
               let x = x.%a in
               %a|}
-                A.id
-                (fqn_longident' path (prefix ^ "loc"))
-                A.id
-                (fqn_longident' path (prefix ^ "desc"))
-                A.expr body
-            in
-            if has_attrs then
-              Exp.sequence (assert_no_attributes ~path ~prefix) body
-            else body
-      in
-      let body =
-        let loc =
-          match wrapper with None -> M.patt "loc" | Some _ -> M.patt "_loc"
+            A.id
+            (fqn_longident' path (prefix ^ "loc"))
+            A.id
+            (fqn_longident' path (prefix ^ "desc"))
+            A.expr body
         in
-        M.expr "T (fun ctx %a x k -> %a)" A.patt loc A.expr body
-      in
-      let body =
-        List.fold_right funcs ~init:body ~f:(fun func acc ->
-            M.expr "fun (T %a) -> %a" A.patt (pvar func) A.expr acc)
-      in
-      M.stri "let %a = %a" A.patt
-        (pvar (function_name_of_id ~prefix cd.pcd_name.txt))
-        A.expr body
+        if has_attrs then Exp.sequence (assert_no_attributes ~path ~prefix) body
+        else body
+  in
+  let body =
+    let loc =
+      match wrapper with None -> M.patt "loc" | Some _ -> M.patt "_loc"
+    in
+    M.expr "T (fun ctx %a x k -> %a)" A.patt loc A.expr body
+  in
+  let body =
+    List.fold_right funcs ~init:body ~f:(fun func acc ->
+        M.expr "fun (T %a) -> %a" A.patt (pvar func) A.expr acc)
+  in
+  M.stri "let %a = %a" A.patt
+    (pvar (function_name_of_id ~prefix cd.pcd_name.txt))
+    A.expr body
 
 let gen_combinator_for_record path ~prefix ~has_attrs lds =
   let fields = List.map lds ~f:(fun ld -> fqn_longident path ld.pld_name.txt) in
@@ -237,13 +258,9 @@ let generate filename =
   dump "ast_pattern_generated" Pprintast.structure st ~ext:".ml"
 
 let args = []
-
 let usage = Printf.sprintf "%s [options] <.ml files>\n" Sys.argv.(0)
 
 let () =
   let fns = ref [] in
   Arg.parse (Arg.align args) (fun fn -> fns := fn :: !fns) usage;
-  try List.iter (List.rev !fns) ~f:generate
-  with exn ->
-    Astlib.Location.report_exception Format.err_formatter exn;
-    exit 2
+  List.iter (List.rev !fns) ~f:generate
